@@ -130,7 +130,7 @@ class Nag_Api extends Horde_Registry_Api
         try {
             $tasklist = $GLOBALS['nag_shares']->getShare($name);
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             throw new Nag_Exception($e);
         }
         if (!$tasklist->hasPermission($GLOBALS['registry']->getAuth(), Horde_Perms::READ)) {
@@ -145,13 +145,25 @@ class Nag_Api extends Horde_Registry_Api
      * @param string $name        Task list name.
      * @param string $description Task list description.
      * @param string $color       Task list color.
+     * @param array  $params      Any addtional parameters needed. @since 4.2.1
+     *     - synchronize:   (boolean) If true, add task list to the list of
+     *                                task lists to syncronize.
+     *                      DEFAULT: false (do not add to the list).
      *
-     * @return integer  The new tasklist's id.
+     * @return string  The new tasklist's id.
      */
-    public function addTasklist($name, $description = '', $color = '')
+    public function addTasklist($name, $description = '', $color = '', array $params = array())
     {
         $tasklist = Nag::addTasklist(array('name' => $name, 'description' => $description, 'color' => $color));
-        return $tasklist->getName();
+
+        $name = $tasklist->getName();
+        if (!empty($params['synchronize'])) {
+            $sync = @unserialize($prefs->getValue('sync_lists'));
+            $sync[] = $name;
+            $prefs->setValue('sync_lists', serialize($sync));
+        }
+
+        return $name;
     }
 
     /**
@@ -165,7 +177,7 @@ class Nag_Api extends Horde_Registry_Api
         try {
             $tasklist = $GLOBALS['nag_shares']->getShare($name);
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             throw new Nag_Exception($e);
         }
 
@@ -335,15 +347,15 @@ class Nag_Api extends Horde_Registry_Api
             //
             $tasklist = substr($parts[1], 0, -4);
             if (!Nag::hasPermission($tasklist, Horde_Perms::READ)) {
-                throw new Nag_Exception(_("Invalid tasklist file requested."), 404);
+                throw new Nag_Exception(_("Invalid task list file requested."), 404);
             }
             $ical_data = $this->exportTasklist($tasklist, 'text/calendar');
-            $result = array('data'          => $ical_data,
+            return array(
+                'data'          => $ical_data,
                 'mimetype'      => 'text/calendar',
                 'contentlength' => strlen($ical_data),
-                'mtime'         => $_SERVER['REQUEST_TIME']);
-
-            return $result;
+                'mtime'         => $_SERVER['REQUEST_TIME']
+            );
 
         } elseif (count($parts) == 2) {
             //
@@ -351,7 +363,7 @@ class Nag_Api extends Horde_Registry_Api
             // of items and represent them as files within the directory.
             //
             if (!Nag::hasPermission($parts[1], Horde_Perms::READ)) {
-                throw new Nag_Exception(_("Invalid tasklist requested."), 404);
+                throw new Nag_Exception(_("Invalid task list requested."), 404);
             }
             $storage = $GLOBALS['injector']->getInstance('Nag_Factory_Driver')->create($parts[1]);
             try {
@@ -469,13 +481,13 @@ class Nag_Api extends Horde_Registry_Api
                     $content_type = 'text/calendar';
                 }
             } else {
-                throw new Nag_Exception(_("Invalid tasklist name supplied."), 403);
+                throw new Nag_Exception(_("Invalid task list name supplied."), 403);
             }
 
         if (!Nag::hasPermission($tasklist, Horde_Perms::EDIT)) {
             // FIXME: Should we attempt to create a tasklist based on the filename
             // in the case that the requested tasklist does not exist?
-            throw new Nag_Exception(_("Tasklist does not exist or no permission to edit"), 403);
+            throw new Nag_Exception(_("Task list does not exist or no permission to edit"), 403);
         }
 
         // Store all currently existings UIDs. Use this info to delete UIDs not
@@ -604,7 +616,7 @@ class Nag_Api extends Horde_Registry_Api
             // Allow users to delete tasklists but not create them via WebDAV will
             // be more confusing than helpful.  They are, however, still able to
             // delete individual task items within the tasklist folder.
-            throw Nag_Exception(_("Deleting entire tasklists is not supported."), 403);
+            throw Nag_Exception(_("Deleting entire task lists is not supported."), 403);
             // To re-enable the functionality just remove this if {} block.
         }
 
@@ -617,7 +629,7 @@ class Nag_Api extends Horde_Registry_Api
         if (!(count($parts) == 2 || count($parts) == 3) ||
             !Nag::hasPermission($tasklistID, Horde_Perms::DELETE)) {
 
-            throw new Nag_Exception(_("Tasklist does not exist or no permission to delete"), 403);
+            throw new Nag_Exception(_("Task list does not exist or no permission to delete"), 403);
         }
 
         /* Create a Nag storage instance. */
@@ -635,7 +647,7 @@ class Nag_Api extends Horde_Registry_Api
             try {
                 $storage->deleteAll();
             } catch (Nag_Exception $e) {
-                throw new Nag_Exception(sprintf(_("Unable to delete tasklist \"%s\": %s"), $tasklistID, $e->getMessage()), 500);
+                throw new Nag_Exception(sprintf(_("Unable to delete task list \"%s\": %s"), $tasklistID, $e->getMessage()), 500);
             }
 
             // Remove share and all groups/permissions.
@@ -749,30 +761,32 @@ class Nag_Api extends Horde_Registry_Api
      * @param boolean $isModSeq          If true, $timestamp and $end are
      *                                   modification sequences and not
      *                                   timestamps. @since 4.1.1
+     * @param string|array $tasklists    The sources to check. @since 4.2.0
      *
      * @return array  An hash with 'add', 'modify' and 'delete' arrays.
      */
-    public function getChanges($start, $end, $isModSeq = false)
+    public function getChanges($start, $end, $isModSeq = false, $tasklists = null)
     {
         return array(
-            'add' => $this->listBy('add', $start, null, $end, $isModSeq),
-            'modify' => $this->listBy('modify', $start, null, $end, $isModSeq),
-            'delete' => $this->listBy('delete', $start, null, $end, $isModSeq));
+            'add' => $this->listBy('add', $start, $tasklists, $end, $isModSeq),
+            'modify' => $this->listBy('modify', $start, $tasklists, $end, $isModSeq),
+            'delete' => $this->listBy('delete', $start, $tasklists, $end, $isModSeq));
     }
 
     /**
      * Return all changes occuring between the specified modification
      * sequences.
      *
-     * @param integer $start  The starting modseq.
-     * @param integer $end    The ending modseq.
+     * @param integer $start             The starting modseq.
+     * @param integer $end               The ending modseq.
+     * @param string|array $tasklists    The sources to check. @since 4.2.0
      *
      * @return array  The changes @see getChanges()
      * @since 4.1.1
      */
-    public function getChangesByModSeq($start, $end)
+    public function getChangesByModSeq($start, $end, $tasklists = null)
     {
-        return $this->getChanges($start, $end, true, true);
+        return $this->getChanges($start, $end, true, $tasklists);
     }
 
     /**
@@ -813,12 +827,18 @@ class Nag_Api extends Horde_Registry_Api
     /**
      * Return the largest modification sequence from the history backend.
      *
+     * @param string $id  Limit the check to this tasklist. @since 4.2.0
+     *
      * @return integer  The modseq.
      * @since 4.1.1
      */
-    public function getHighestModSeq()
+    public function getHighestModSeq($id = null)
     {
-        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq('nag');
+        $parent = 'nag';
+        if (!empty($id)) {
+            $parent .= ':' . $id;
+        }
+        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq($parent);
     }
 
     /**
@@ -969,7 +989,7 @@ class Nag_Api extends Horde_Registry_Api
         try {
             $share = $GLOBALS['nag_shares']->getShare($tasklist_id);
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             throw new Nag_Exception($e);
         }
         $task = Nag::getTask($tasklist_id, $task_id);
@@ -1078,6 +1098,7 @@ class Nag_Api extends Horde_Registry_Api
         $tasks = Nag::listTasks(array(
             'tasklists' => array($tasklist),
             'completed' => Nag::VIEW_ALL,
+            'external' => false,
             'include_tags' => true));
 
         $version = '2.0';
@@ -1258,9 +1279,10 @@ class Nag_Api extends Horde_Registry_Api
         );
         $result = array();
         $tasks->reset();
-        $last_week = time() - 7 * 86400;
+        $last_week = $_SERVER['REQUEST_TIME'] - 7 * 86400;
         while ($task = $tasks->each()) {
-            if ($task->completed && $task->completed_date < $last_week) {
+            if (($task->completed && $task->completed_date < $last_week) ||
+                ($task->start && $task->start > $_SERVER['REQUEST_TIME'])) {
                 continue;
             }
             $result[$task->id] = array(
@@ -1404,4 +1426,115 @@ class Nag_Api extends Horde_Registry_Api
         $storage->modify($timeobject['id'], $info);
     }
 
+    /**
+     * Returns a list of available sources.
+     *
+     * @param boolean $writeable  If true, limits to writeable sources.
+     * @param boolean $sync_only  Only include synchable address books.
+     *
+     * @return array  An array of the available sources. Keys are source IDs,
+     *                values are source titles.
+     * @since 4.2.0
+     */
+    public function sources($writeable = false, $sync_only = false)
+    {
+        $out = array();
+
+        foreach (Nag::listTasklists(false, $writeable ? Horde_Perms::EDIT : Horde_Perms::READ, false) as $key => $val) {
+            $out[$key] = $val->get('name');
+        }
+
+        if ($sync_only) {
+            $syncable = Nag::getSyncLists();
+            $out = array_intersect_key($out, array_flip($syncable));
+        }
+
+        return $out;
+    }
+
+    /**
+     * Retrieve the UID for the current user's default tasklist.
+     *
+     * @return string  UID.
+     * @since 4.2.0
+     */
+    public function getDefaultShare()
+    {
+        return Nag::getDefaultTasklist(Horde_Perms::EDIT);
+    }
+
+    /**
+     * Retrieve the list of used tag_names, tag_ids and the total number
+     * of resources that are linked to that tag.
+     *
+     * @param array $tags  An optional array of tag_ids. If omitted, all tags
+     *                     will be included.
+     *
+     * @return array  An array containing tag_name, and total
+     */
+    public function listTagInfo($tags = null, $user = null)
+    {
+        return $GLOBALS['injector']->getInstance('Nag_Tagger')
+            ->getTagInfo($tags, 500, null, $user);
+    }
+
+    /**
+     * SearchTags API:
+     * Returns an application-agnostic array (useful for when doing a tag search
+     * across multiple applications)
+     *
+     * The 'raw' results array can be returned instead by setting $raw = true.
+     *
+     * @param array $names           An array of tag_names to search for.
+     * @param integer $max           The maximum number of resources to return.
+     * @param integer $from          The number of the resource to start with.
+     * @param string $resource_type  The resource type [bookmark, '']
+     * @param string $user           Restrict results to resources owned by $user.
+     * @param boolean $raw           Return the raw data?
+     *
+     * @return array An array of results:
+     * <pre>
+     *  'title'    - The title for this resource.
+     *  'desc'     - A terse description of this resource.
+     *  'view_url' - The URL to view this resource.
+     *  'app'      - The Horde application this resource belongs to.
+     *  'icon'     - URL to an image.
+     * </pre>
+     */
+    public function searchTags($names, $max = 10, $from = 0,
+                               $resource_type = '', $user = null, $raw = false)
+    {
+        // TODO: $max, $from, $resource_type not honored
+        global $injector, $registry;
+
+        $results = $injector
+            ->getInstance('Nag_Tagger')
+            ->search(
+                $names,
+                array('user' => $user));
+
+        // Check for error or if we requested the raw data array.
+        if ($raw) {
+            return $results;
+        }
+
+        $return = array();
+        $redirectUrl = Horde::url('redirect.php');
+        foreach ($results as $task_id) {
+            try {
+                $task = $injector->getInstance('Nag_Factory_Driver')
+                    ->create(null)
+                    ->getByUID($task_id);
+                $return[] = array(
+                    'title' => $task->name,
+                    'desc' => $task->description,
+                    'view_url' => $redirectUrl->add('b', $task->id),
+                    'app' => 'nag'
+                );
+            } catch (Exception $e) {
+            }
+        }
+
+        return $return;
+    }
 }
