@@ -13,7 +13,7 @@
 
 /* Determine the base directories. */
 if (!defined('NAG_BASE')) {
-    define('NAG_BASE', __DIR__ . '/..');
+    define('NAG_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -22,7 +22,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(NAG_BASE . '/config/horde.local.php')) {
         include NAG_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', NAG_BASE . '/..');
+        define('HORDE_BASE', realpath(NAG_BASE . '/..'));
     }
 }
 
@@ -30,6 +30,8 @@ if (!defined('HORDE_BASE')) {
  * Horde_Registry_Application::). */
 require_once HORDE_BASE . '/lib/core.php';
 
+use Sabre\CalDAV;
+
 class Nag_Application extends Horde_Registry_Application
 {
     /**
@@ -41,7 +43,7 @@ class Nag_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (4.1.3)';
+    public $version = 'H5 (4.2.1)';
 
     /**
      * Global variables defined:
@@ -146,6 +148,19 @@ class Nag_Application extends Horde_Registry_Application
                 'label' => _("Create a new Task List"),
             );
         }
+        if ($GLOBALS['registry']->isAdmin()) {
+            $sidebar->containers['system'] = array(
+                'header' => array(
+                    'id' => 'nag-toggle-system',
+                    'label' => _("System Task Lists"),
+                    'collapsed' => true,
+                ),
+            );
+            $sidebar->containers['system']['header']['add'] = array(
+                'url' => Horde::url('tasklists/create.php')->add('system', 1),
+                'label' => _("Create a new System Task List"),
+            );
+        }
         $sidebar->containers['shared'] = array(
             'header' => array(
                 'id' => 'nag-toggle-shared',
@@ -153,7 +168,7 @@ class Nag_Application extends Horde_Registry_Application
                 'collapsed' => true,
             ),
         );
-        foreach (Nag::listTaskLists(false, Horde_Perms::SHOW, false) as $name => $tasklist) {
+        foreach (Nag::listTasklists(false, Horde_Perms::SHOW, false) as $name => $tasklist) {
             $url = $list->add(array(
                 'display_tasklist' => $name,
                 'actionID' => in_array($name, $display_tasklists)
@@ -168,7 +183,9 @@ class Nag_Application extends Horde_Registry_Application
                 'edit' => $edit->add('t', $tasklist->getName()),
                 'type' => 'checkbox',
             );
-            if ($tasklist->get('owner') == $user) {
+            if (is_null($tasklist->get('owner'))) {
+                $sidebar->addRow($row, 'system');
+            } elseif ($tasklist->get('owner') == $user) {
                 $sidebar->addRow($row, 'my');
             } else {
                 $sidebar->addRow($row, 'shared');
@@ -202,7 +219,7 @@ class Nag_Application extends Horde_Registry_Application
             $shares = $GLOBALS['nag_shares']
                 ->listShares($user, array('attributes' => $user));
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
             throw new Nag_Exception($e);
         }
 
@@ -213,7 +230,7 @@ class Nag_Application extends Horde_Registry_Application
             try {
                 $GLOBALS['nag_shares']->removeShare($share);
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e, 'NOTICE');
+                Horde::log($e, 'NOTICE');
                 $error = true;
             }
         }
@@ -225,7 +242,7 @@ class Nag_Application extends Horde_Registry_Application
                $share->removeUser($user);
             }
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e, 'NOTICE');
+            Horde::log($e, 'NOTICE');
             $error = true;
         }
 
@@ -294,7 +311,7 @@ class Nag_Application extends Horde_Registry_Application
 
         switch ($params['id']) {
         case 'menu':
-            $add = Horde::url('task.php')->add('actionID', 'add_task');
+            $add = Horde::url('task.php', true)->add('actionID', 'add_task');
 
             $tree->addNode(array(
                 'id' => $parent . '__new',
@@ -307,7 +324,7 @@ class Nag_Application extends Horde_Registry_Application
                 )
             ));
 
-            foreach (Nag::listTasklists(false, Horde_Perms::EDIT) as $name => $tasklist) {
+            foreach (Nag::listTasklists(false, Horde_Perms::EDIT, false) as $name => $tasklist) {
                 $tree->addNode(array(
                     'id' => $parent . $name . '__new',
                     'parent' => $parent . '__new',
@@ -423,11 +440,8 @@ class Nag_Application extends Horde_Registry_Application
     public function davGetCollections($user)
     {
         $opts = array('perm' => Horde_Perms::SHOW);
-        if ($user != '-system-') {
-            $opts['attributes'] = $user;
-        }
         $shares = $GLOBALS['nag_shares']
-            ->listShares($GLOBALS['registry']->getAuth(), $opts);
+            ->listShares($user, $opts);
         $dav = $GLOBALS['injector']
             ->getInstance('Horde_Dav_Storage');
         $tasklists = array();
@@ -442,13 +456,18 @@ class Nag_Application extends Horde_Registry_Application
             $tasklists[] = array(
                 'id' => $id,
                 'uri' => $id,
+                '{' . CalDAV\Plugin::NS_CALENDARSERVER . '}shared-url' =>
+                    Nag::getUrl(Nag::DAV_CALDAV, $share),
                 'principaluri' => 'principals/' . $user,
+                '{http://sabredav.org/ns}owner-principal' =>
+                    'principals/' . ($share->get('owner') ?: '-system-'),
                 '{DAV:}displayname' => Nag::getLabel($share),
                 '{urn:ietf:params:xml:ns:caldav}calendar-description' =>
                     $share->get('desc'),
                 '{http://apple.com/ns/ical/}calendar-color' =>
                     $share->get('color'),
                 '{urn:ietf:params:xml:ns:caldav}supported-calendar-component-set' => new Sabre\CalDAV\Property\SupportedCalendarComponentSet(array('VTODO')),
+                '{http://sabredav.org/ns}read-only' => !$share->hasPermission($user, Horde_Perms::EDIT),
             );
         }
         return $tasklists;
