@@ -104,6 +104,7 @@ abstract class Nag_Driver
      *     - private: (OPTIONAL, boolean) Whether the task is private.
      *     - owner: (OPTIONAL, string) The owner of the event.
      *     - assignee: (OPTIONAL, string) The assignee of the event.
+     *     - completed_date: (OPTIONAL, integer) The task's completion date.
      *     - recurrence: (OPTIONAL, Horde_Date_Recurrence|array) Recurrence
      *                   information.
      *
@@ -125,6 +126,7 @@ abstract class Nag_Driver
                   'private' => false,
                   'owner' => $GLOBALS['registry']->getAuth(),
                   'assignee' => null,
+                  'completed_date' => 0,
                   'recurrence' => null),
             $task
         );
@@ -140,7 +142,7 @@ abstract class Nag_Driver
                           array('action' => 'add'),
                           true);
         } catch (Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
 
         /* Log completion status changes. */
@@ -150,7 +152,7 @@ abstract class Nag_Driver
                               array('action' => 'complete'),
                               true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -160,7 +162,6 @@ abstract class Nag_Driver
         /* Add an alarm if necessary. */
         if (!empty($task->alarm) &&
             ($alarm = $task->toAlarm())) {
-            $alarm['start'] = new Horde_Date($alarm['start']);
             $GLOBALS['injector']->getInstance('Horde_Alarm')->set($alarm);
         }
 
@@ -216,6 +217,7 @@ abstract class Nag_Driver
          * because this might be a recurring task, and marking the
          * task complete might only shift the due date to the next
          * recurrence. */
+        $task_completed = $task->completed;
         if (isset($properties['completed']) &&
             $properties['completed'] != $task->completed) {
             if (isset($properties['recurrence'])) {
@@ -236,6 +238,10 @@ abstract class Nag_Driver
 
         $this->_modify($taskId, array_merge($task->toHash(), $properties));
 
+        // Must persist the original completed value so we can detect the
+        // change when notifications are sent.
+        $task->completed = $task_completed;
+
         $new_task = $this->get($task->id);
         $log_tasklist = $this->_tasklist;
         if (isset($properties['tasklist']) &&
@@ -244,7 +250,7 @@ abstract class Nag_Driver
             try {
                 $share = $GLOBALS['nag_shares']->getShare($task->tasklist);
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e->getMessage(), 'ERR');
+                Horde::log($e->getMessage(), 'ERR');
                 throw new Nag_Exception($e);
             }
 
@@ -256,7 +262,7 @@ abstract class Nag_Driver
             try {
                 $share = $GLOBALS['nag_shares']->getShare($properties['tasklist']);
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e->getMessage(), 'ERR');
+                Horde::log($e->getMessage(), 'ERR');
                 throw new Nag_Exception($e);
             }
 
@@ -274,12 +280,12 @@ abstract class Nag_Driver
                 try {
                     $history->log('nag:' . $task->tasklist . ':' . $task->uid, array('action' => 'delete'), true);
                 } catch (Exception $e) {
-                    Horde::logMessage($e, 'ERR');
+                    Horde::log($e, 'ERR');
                 }
                 try {
                     $history->log('nag:' . $properties['tasklist'] . ':' . $task->uid, array('action' => 'add'), true);
                 } catch (Exception $e) {
-                    Horde::logMessage($e, 'ERR');
+                    Horde::log($e, 'ERR');
                 }
                 $log_tasklist = $properties['tasklist'];
             }
@@ -329,7 +335,7 @@ abstract class Nag_Driver
                         array('action' => 'modify'),
                         true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -346,7 +352,7 @@ abstract class Nag_Driver
                         $attributes,
                         true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -354,7 +360,7 @@ abstract class Nag_Driver
         try {
             $result = Nag::sendNotification('edit', $new_task, $task);
         } catch (Nag_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
     }
 
@@ -379,7 +385,7 @@ abstract class Nag_Driver
             try {
                 $GLOBALS['injector']->getInstance('Horde_History')->log('nag:' . $this->_tasklist . ':' . $task->uid, array('action' => 'delete'), true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -387,7 +393,7 @@ abstract class Nag_Driver
         try {
             $result = Nag::sendNotification('delete', $task);
         } catch (Nag_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
 
         /* Delete alarm if necessary. */
@@ -424,7 +430,7 @@ abstract class Nag_Driver
                     true);
             }
         } catch (Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
     }
 
