@@ -2,7 +2,7 @@
 /**
  * Nag driver classes for the Kolab IMAP server.
  *
- * Copyright 2004-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2004-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -412,10 +412,10 @@ class Nag_Driver_Kolab extends Nag_Driver
                 $start = $t->start;
             }
 
-            if (($completed == Nag::VIEW_INCOMPLETE && ($complete || $start > time())) ||
+            if (($completed == Nag::VIEW_INCOMPLETE && ($complete || $start > $_SERVER['REQUEST_TIME'])) ||
                 ($completed == Nag::VIEW_COMPLETE && !$complete) ||
                 ($completed == Nag::VIEW_FUTURE &&
-                 ($complete || $start == 0 || $start < time())) ||
+                 ($complete || $start == 0 || $start < $_SERVER['REQUEST_TIME'])) ||
                 ($completed == Nag::VIEW_FUTURE_INCOMPLETE && $complete)) {
                 continue;
             }
@@ -482,7 +482,7 @@ class Nag_Driver_Kolab extends Nag_Driver
         $tasks = array();
 
         foreach ($task_list as $task) {
-          if (Horde_Url::uriB64Encode($task['parent']) != $parentId) {
+            if (Horde_Url::uriB64Encode($task['parent']) != $parentId) {
                 continue;
             }
             $t = new Nag_Task($this, $this->_buildTask($task));
