@@ -96,11 +96,11 @@ class Nag_Driver_Sql extends Nag_Driver
             throw new Nag_Exception($e);
         }
         if (!$rows) {
-            throw new Horde_Exception_NotFound("Tasks not found");
+            throw new Horde_Exception_NotFound(_("Tasks not found"));
         }
 
         if (!is_array($taskIds)) {
-            $results = new Nag_Task($this, $this->_buildTask(current($rows)));
+            $results = new Nag_Task($this, $this->_buildTask(reset($rows)));
             $this->_tasklist = $results->tasklist;
         } else {
             $results = new Nag_Task();
@@ -410,7 +410,7 @@ class Nag_Driver_Sql extends Nag_Driver
         switch ($completed) {
         case Nag::VIEW_INCOMPLETE:
             $query .= ' AND task_completed = 0 AND (task_start IS NULL OR task_start = 0 OR task_start < ?)';
-            $values[] = time();
+            $values[] = $_SERVER['REQUEST_TIME'];
             break;
 
         case Nag::VIEW_COMPLETE:
@@ -419,7 +419,7 @@ class Nag_Driver_Sql extends Nag_Driver
 
         case Nag::VIEW_FUTURE:
             $query .= ' AND task_completed = 0 AND task_start > ?';
-            $values[] = time();
+            $values[] = $_SERVER['REQUEST_TIME'];
             break;
 
         case Nag::VIEW_FUTURE_INCOMPLETE:
