@@ -87,6 +87,21 @@ class Nag
      */
     const VIEW_FUTURE_INCOMPLETE = 4;
 
+    /**
+     * WebDAV task list.
+     */
+    const DAV_WEBDAV = 1;
+
+    /**
+     * CalDAV task list.
+     */
+    const DAV_CALDAV = 2;
+
+    /**
+     * CalDAV principal.
+     */
+    const DAV_ACCOUNT = 3;
+
     /**
      *
      * @param integer $seconds
@@ -184,16 +199,21 @@ class Nag
      *                DEFAULT: altsortby pref is used.
      *   - completed: (integer) Which task to retrieve. A Nag::VIEW_* constant.
      *                DEFAULT: show_completed pref is used.
-     *   - sortby: (string)  A Nag::SORT_* constant for the field to sort by.
-     *             DEFAULT: sortby pref is used.
-     *   - sortdir: (string) Direction of sort. NAG::SORT_ASCEND or NAG::SORT_DESCEND.
-     *              DEFAULT: sortdir pref is used.
+     *   - external: (boolean) Whether to include tasks from other applications
+     *               too.
+     *               DEFAULT: true.
+     *   - include_history: (boolean) Autoload created/modified data from
+     *                      Horde_History.
+     *                      DEFAULT: true (Automatically load history data).
      *   - include_tags: (boolean) Autoload all tags.
      *                   DEFAULT: false (Tags are lazy loaded as needed.)
+     *   - sortby: (string)  A Nag::SORT_* constant for the field to sort by.
+     *             DEFAULT: sortby pref is used.
+     *   - sortdir: (string) Direction of sort. NAG::SORT_ASCEND or
+     *              NAG::SORT_DESCEND.
+     *              DEFAULT: sortdir pref is used.
      *   - tasklists: (array) An array of tasklists to include.
      *                DEFAULT: Use $GLOBALS['display_tasklists'];
-     *   - include_history: (boolean) Autoload created/modified data from Horde_History.
-     *                      DEFAULT: true (Automatically load history data).
      *
      * @return Nag_Task  A list of the requested tasks.
      */
@@ -237,31 +257,47 @@ class Nag
         // Process all tasks.
         $tasks->process();
 
-        if ($options['external']) {
-            // We look for registered apis that support listAs(taskHash).
-            $apps = @unserialize($prefs->getValue('show_external'));
-            if (is_array($apps)) {
-                foreach ($apps as $app) {
-                    if ($app != 'nag' &&
-                        $registry->hasMethod('getListTypes', $app)) {
-                        try {
-                            $types = $registry->callByPackage($app, 'getListTypes');
-                        } catch (Horde_Exception $e) {
+        if ($options['external'] &&
+            ($apps = @unserialize($prefs->getValue('show_external'))) &&
+            is_array($apps)) {
+            foreach ($apps as $app) {
+                // We look for registered apis that support listAs(taskHash).
+                if ($app == 'nag' ||
+                    !$registry->hasMethod('getListTypes', $app)) {
+                    continue;
+                }
+                try {
+                    $types = $registry->callByPackage($app, 'getListTypes');
+                } catch (Horde_Exception $e) {
+                    continue;
+                }
+                if (empty($types['taskHash'])) {
+                    continue;
+                }
+
+                try {
+                    $newtasks = $registry->callByPackage($app, 'listAs', array('taskHash'));
+                    foreach ($newtasks as $task) {
+                        $task['tasklist_id'] = '**EXTERNAL**';
+                        $task['tasklist_name'] = $registry->get('name', $app);
+                        $task = new Nag_Task(null, $task);
+                        if (($options['completed'] == Nag::VIEW_INCOMPLETE &&
+                             ($task->completed ||
+                              $task->start > $_SERVER['REQUEST_TIME'])) ||
+                            ($options['completed'] == Nag::VIEW_COMPLETE &&
+                             !$task->completed) ||
+                            ($options['completed'] == Nag::VIEW_FUTURE &&
+                             ($task->completed ||
+                              !$task->start ||
+                              $task->start < $_SERVER['REQUEST_TIME'])) ||
+                            ($options['completed'] == Nag::VIEW_FUTURE_INCOMPLETE &&
+                             $task->completed)) {
                             continue;
                         }
-                        if (!empty($types['taskHash'])) {
-                            try {
-                                $newtasks = $registry->callByPackage($app, 'listAs', array('taskHash'));
-                                 foreach ($newtasks as $task) {
-                                    $task['tasklist_id'] = '**EXTERNAL**';
-                                    $task['tasklist_name'] = $registry->get('name', $app);
-                                    $tasks->add(new Nag_Task(null, $task));
-                                }
-                            } catch (Horde_Exception $e) {
-                                Horde::logMessage($newtasks, 'ERR');
-                            }
-                        }
+                        $tasks->add($task);
                     }
+                } catch (Horde_Exception $e) {
+                    Horde::log($e);
                 }
             }
         }
@@ -430,15 +466,9 @@ class Nag
     /**
      * Lists all task lists a user has access to.
      *
-     * This method takes the $conf['share']['hidden'] setting into account. If
-     * this setting is enabled, even if requesting permissions different than
-     * SHOW, it will only return calendars that the user owns or has SHOW
-     * permissions for. For checking individual calendar's permissions, use
-     * hasPermission() instead.
-     *
-     * @param boolean $owneronly  Only return tasklists that this user owns?
+     * @param boolean $owneronly  Only return task lists that this user owns?
      *                            Defaults to false.
-     * @param integer $permission The permission to filter tasklists by.
+     * @param integer $permission The permission to filter task lists by.
      * @param boolean $smart      Include SmartLists in the results.
      *
      * @return array  The task lists.
@@ -464,9 +494,14 @@ class Nag
                 array('perm' => $permission,
                       'attributes' => $att,
                       'sort_by' => 'name'));
-
+            if ($GLOBALS['registry']->isAdmin()) {
+                $tasklists = array_merge(
+                    $tasklists,
+                    $GLOBALS['nag_shares']->listSystemShares()
+                );
+            }
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             return array();
         }
 
@@ -480,7 +515,7 @@ class Nag
                     }
                 } catch (Horde_Exception_NotFound $e) {
                 } catch (Horde_Share_Exception $e) {
-                    Horde::logMessage($e);
+                    Horde::log($e);
                     return array();
                 }
             }
@@ -532,13 +567,14 @@ class Nag
             ->getInstance('Nag_Factory_Tasklists')
             ->create()
             ->getDefaultShare();
-        if (isset($tasklists[$default_tasklist])) {
-            $GLOBALS['prefs']->setValue('default_tasklist', $default_tasklist);
-            return $default_tasklist;
+
+        if (!isset($tasklists[$default_tasklist])) {
+            reset($tasklists);
+            $default_tasklist = key($tasklists);
         }
 
-        reset($tasklists);
-        return key($tasklists);
+        $GLOBALS['prefs']->setValue('default_tasklist', $default_tasklist);
+        return $default_tasklist;
     }
 
     /**
@@ -662,6 +698,65 @@ class Nag
         return $label;
     }
 
+    /**
+     * Returns a DAV URL to be used for a task list.
+     *
+     * @param integer $type       A Nag::DAV_* constant.
+     * @param Horde_Share_Object  A task list.
+     *
+     * @return string  The task list's URL.
+     * @throws Horde_Exception
+     */
+    public static function getUrl($type, $tasklist)
+    {
+        global $conf, $injector, $registry;
+
+        $url = $registry->get('webroot', 'horde');
+        $rewrite = isset($conf['urls']['pretty']) &&
+            $conf['urls']['pretty'] == 'rewrite';
+
+        switch ($type) {
+        case Nag::DAV_WEBDAV:
+            if ($rewrite) {
+                $url .= '/rpc/nag/';
+            } else {
+                $url .= '/rpc.php/nag/';
+            }
+            $url = Horde::url($url, true, -1)
+                . ($tasklist->get('owner')
+                   ? $tasklist->get('owner')
+                   : '-system-')
+                . '/' . $tasklist->getName() . '.ics';
+            break;
+
+        case Nag::DAV_CALDAV:
+            if ($rewrite) {
+                $url .= '/rpc/calendars/';
+            } else {
+                $url .= '/rpc.php/calendars/';
+            }
+            $url = Horde::url($url, true, -1)
+                . $registry->getAuth()
+                . '/'
+                . $injector->getInstance('Horde_Dav_Storage')
+                    ->getExternalCollectionId($tasklist->getName(), 'tasks')
+                . '/';
+            break;
+
+        case Nag::DAV_ACCOUNT:
+            if ($rewrite) {
+                $url .= '/rpc/';
+            } else {
+                $url .= '/rpc.php/';
+            }
+            $url = Horde::url($url, true, -1)
+                . 'principals/'. $registry->getAuth() . '/';
+            break;
+        }
+
+        return $url;
+    }
+
     /**
      * Returns a random CSS color.
      *
@@ -897,12 +992,11 @@ class Nag
 
         if (empty($GLOBALS['conf']['alarms']['driver'])) {
             // Get any alarms in the next hour.
-            $now = time();
             try {
-                $alarmList = self::listAlarms($now);
+                $alarmList = self::listAlarms($_SERVER['REQUEST_TIME']);
                 $messages = array();
                 foreach ($alarmList as $task) {
-                    $differential = $task->due - $now;
+                    $differential = $task->due - $_SERVER['REQUEST_TIME'];
                     $key = $differential;
                     while (isset($messages[$key])) {
                         $key++;
@@ -920,7 +1014,7 @@ class Nag
                     $notification->push($message[0], $message[1]);
                 }
             } catch (Nag_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 $notification->push($e->getMessage(), 'horde.error');
             }
         }
@@ -955,7 +1049,7 @@ class Nag
         try {
             $share = $GLOBALS['nag_shares']->getShare($task->tasklist);
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             throw new Nag_Exception($e);
         }
 
@@ -978,7 +1072,7 @@ class Nag
             try {
                 $group_users = $groups->listUsers($group);
             } catch (Horde_Group_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 continue;
             }
 
@@ -1151,8 +1245,7 @@ class Nag
                     $mail->clearRecipients();
                     $mail->addRecipients($df_recipients);
 
-                    Horde::logMessage(sprintf('Sending event notifications for %s to %s',
-                                              $task->name, implode(', ', $df_recipients)), 'INFO');
+                    Horde::log(sprintf('Sending event notifications for %s to %s', $task->name, implode(', ', $df_recipients)), 'INFO');
                     $mail->send($GLOBALS['injector']->getInstance('Horde_Mail'));
                 }
             }
@@ -1277,7 +1370,7 @@ class Nag
         case 'show':
             if ($mode == 'read') {
                 $display_tasklists = unserialize($prefs->getValue('display_tasklists'));
-                return in_array($tasklist, $display_tasklists) ? $vals : false;;
+                return in_array($tasklist, $display_tasklists) ? $vals : false;
             }
         }
 
@@ -1585,9 +1678,9 @@ class Nag
     }
 
     /**
-     * Returns the calendars that should be used for syncing.
+     * Returns the tasklists that should be used for syncing.
      *
-     * @return array  An array of calendar ids
+     * @return array  An array of task list ids
      */
     static public function getSyncLists()
     {
