@@ -84,7 +84,7 @@ class Nag_Task
     public $recurrence;
 
     /**
-     * The task priority.
+     * The task priority from 1 = highest to 5 = lowest.
      *
      * @var integer
      */
@@ -140,7 +140,7 @@ class Nag_Task
     public $modifiedby;
 
     /**
-     * The task alarm threshold.
+     * The task alarm threshold in minutes.
      *
      * @var integer
      */
@@ -708,15 +708,23 @@ class Nag_Task
      *
      * @param integer $indent  The indention level of the tasks.
      */
-    public function process($indent = 0)
+    public function process($indent = null)
     {
+        global $conf;
+
         /* Link cache. */
         static $view_url_list, $task_url_list;
 
         /* Set indention. */
+        if (is_null($indent)) {
+            $indent = 0;
+            if ($parent = $this->getParent()) {
+                $indent = $parent->indent + 1;
+            }
+        }
         $this->indent = $indent;
         if ($this->id) {
-            ++$indent;
+            $indent++;
         }
 
         /* Process children. */
@@ -750,7 +758,15 @@ class Nag_Task
         $this->view_link = $view_url_list[$this->tasklist]->copy()->add('task', $this->id);
 
         $task_url_task = $task_url_list[$this->tasklist]->copy()->add('task', $this->id);
-        $this->complete_link = Horde::url('t/complete')->add(array('url' => Horde::url('list.php'), 'task' => $this->id, 'tasklist' => $this->tasklist));
+        $this->complete_link = Horde::url(
+            $conf['urls']['pretty'] == 'rewrite'
+                ? 't/complete'
+                : 'task/complete.php'
+            )->add(array(
+                'url' => Horde::url('list.php'),
+                'task' => $this->id,
+                'tasklist' => $this->tasklist
+            ));
         $this->edit_link = $task_url_task->copy()->add('actionID', 'modify_task');
         $this->delete_link = $task_url_task->copy()->add('actionID', 'delete_task');
     }
@@ -942,6 +958,7 @@ class Nag_Task
     {
         $json = new stdClass;
         $json->l = $this->tasklist;
+        $json->i = $this->indent;
         $json->n = $this->name;
         if ($this->desc) {
             //TODO: Get the proper amount of characters, and cut by last
@@ -965,6 +982,7 @@ class Nag_Task
         if ($full) {
             // @todo: do we really need all this?
             $json->id = $this->id;
+            $json->p = $this->parent_id;
             $json->de = $this->desc;
             if ($this->due) {
                 $date = new Horde_Date($this->due);
@@ -994,7 +1012,7 @@ class Nag_Task
             try {
                 $share = $GLOBALS['nag_shares']->getShare($this->tasklist);
             } catch (Horde_Share_exception $e) {
-                Horde::logMessage($e->getMessage(), 'ERR');
+                Horde::log($e->getMessage(), 'ERR');
                 throw new Nag_Exception($e);
             }
             $json->pe = $share->hasPermission($GLOBALS['registry']->getAuth(), Horde_Perms::EDIT);
@@ -1063,6 +1081,11 @@ class Nag_Task
 
             $methods['mail']['mimepart'] = Nag::buildMimeMessage($view, 'mail', $image);
         }
+
+        if (isset($methods['desktop'])) {
+            $methods['desktop']['url'] = Horde::url('view.php', true)->add('tasklist', $this->tasklist)->add('task', $this->id)->toString(true, true);
+        }
+
         return array(
             'id' => $this->uid,
             'user' => $user,
@@ -1133,7 +1156,7 @@ class Nag_Task
 
         if ($this->completed) {
             $vTodo->setAttribute('STATUS', 'COMPLETED');
-            $vTodo->setAttribute('COMPLETED', $this->completed_date ? $this->completed_date : time());
+            $vTodo->setAttribute('COMPLETED', $this->completed_date ? $this->completed_date : $_SERVER['REQUEST_TIME']);
         } else {
             if ($v1) {
                 $vTodo->setAttribute('STATUS', 'NEEDS ACTION');
@@ -1200,14 +1223,20 @@ class Nag_Task
             $body = new Horde_ActiveSync_Message_AirSyncBaseBody();
             $body->type = Horde_ActiveSync::BODYPREF_TYPE_PLAIN;
             if (isset($bp[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'])) {
-                if (Horde_String::length($this->desc) > $bp[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize']) {
-                    $body->data = Horde_String::substr($this->desc, 0, $bp[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize']);
-                    $body->truncated = 1;
-                } else {
-                    $body->data = $this->desc;
-                }
-                $body->estimateddatasize = Horde_String::length($this->desc);
+                $truncation = $bp[Horde_ActiveSync::BODYPREF_TYPE_PLAIN]['truncationsize'];
+            } elseif (isset($bp[Horde_ActiveSync::BODYPREF_TYPE_HTML])) {
+                $truncation = $bp[Horde_ActiveSync::BODYPREF_TYPE_HTML]['truncationsize'];
+                $this->desc = Horde_Text_Filter::filter($this->desc, 'Text2html', array('parselevel' => Horde_Text_Filter_Text2html::MICRO));
+            } else {
+                $truncation = false;
             }
+            if ($truncation && Horde_String::length($this->desc) > $truncation) {
+                $body->data = Horde_String::substr($this->desc, 0, $truncation);
+                $body->truncated = 1;
+            } else {
+                $body->data = $this->desc;
+            }
+            $body->estimateddatasize = Horde_String::length($this->desc);
             $message->airsyncbasebody = $body;
         } else {
             $message->body = $this->desc;
@@ -1384,7 +1413,11 @@ class Nag_Task
 
         /* Notes and Title */
         if ($message->getProtocolVersion() >= Horde_ActiveSync::VERSION_TWELVE) {
-            $this->desc = $message->airsyncbasebody->data;
+            if ($message->airsyncbasebody->type == Horde_ActiveSync::BODYPREF_TYPE_HTML) {
+                $this->desc = Horde_Text_Filter::filter($message->airsyncbasebody->data, 'Html2text');
+            } else {
+                $this->desc = $message->airsyncbasebody->data;
+            }
         } else {
             $this->desc = $message->body;
         }
