@@ -2,7 +2,7 @@
 /**
  * Task list view.
  *
- * Copyright 2011-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (BSD). If you did not
  * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
@@ -54,6 +54,13 @@ class Nag_View_List
      */
     protected $_vars;
 
+    /**
+     * Flag to indicate if we have a search.
+     *
+     * @var boolean
+     */
+    protected $_haveSearch = false;
+
     /**
      * Const'r
      *
@@ -121,6 +128,9 @@ class Nag_View_List
         $view->dateFormat = $prefs->getValue('date_format');
         $view->columns = @unserialize($prefs->getValue('tasklist_columns'));
         $view->smartShare = $this->_smartShare;
+        $view->haveSearch = $this->_haveSearch;
+        $view->tab_name = $this->_vars->get('tab_name', $prefs->getValue('show_completed'));
+
         if (empty($view->columns)) {
             $view->columns = array();
         }
@@ -176,6 +186,14 @@ class Nag_View_List
             $action = $this->_vars->actionID;
         }
         switch ($action) {
+        case 'search_return':
+            if ($this->_vars->list) {
+                return $this->_handleActions('smart');
+            }
+            if ($search = $GLOBALS['session']->get('nag', 'search', Horde_Session::TYPE_OBJECT)) {
+                $search->getVars($this->_vars);
+            }
+            // Fall through
         case 'search_tasks':
             if ($this->_vars->deletebutton) {
                 $this->_doDeleteSmartList();
@@ -211,6 +229,7 @@ class Nag_View_List
             $this->_title = $list->get('name');
             $this->_smartShare = $list;
             $this->_loadTasks($lists);
+            $this->_haveSearch = true;
             break;
         default:
             // If we have an active tag browse, use it.
@@ -341,7 +360,8 @@ class Nag_View_List
                 $this->_title .= sprintf(_("and tagged with %s"), $this->_vars->search_tags);
             }
         }
-
+        $GLOBALS['session']->set('nag', 'search', $search, Horde_Session::TYPE_OBJECT);
+        $this->_haveSearch = true;
         $this->_tasks = $tasks;
     }
 
